#include <FastTrig.h>
inline float tsin(float t) { return isin(360*t); }

#include <ESP_I2S.h>
I2SClass i2s;
#define SAMPLE_RATE 16000

struct mixer_proc {
  float (*proc)(uint64_t sample_count, void* data);
};

void task_i2s_audio(void *param) {
  const int i2s_bclk_pin = 4;
  const int i2s_ws_pin = 5;
  const int i2s_dout_pin = 18;
  i2s.setPins(i2s_bclk_pin, i2s_ws_pin, i2s_dout_pin);
  i2s_mode_t mode = I2S_MODE_STD;
  i2s_data_bit_width_t bits_per_sample = I2S_DATA_BIT_WIDTH_32BIT;
  i2s_slot_mode_t slot_mode = I2S_SLOT_MODE_STEREO;
  if (!i2s.begin(mode, SAMPLE_RATE, bits_per_sample, slot_mode)){
    Serial.println("Failed to initialize I2S!");
    while (1);
  }
  Serial.println("I2S task started");

  mixer_proc* p = (mixer_proc*)param;
  uint64_t sample_count = 0;
  for (;;) {
    float value = p->proc(sample_count++, p);
    value *= INT32_MAX;
    int32_t sample[2] = { value, value };
    i2s.write((uint8_t *)sample, sizeof(sample));
  }
}

void init_audio(void* param) {
  TaskHandle_t i2s_task;
  xTaskCreatePinnedToCore(task_i2s_audio, /* Task function. */
                          "I2S Audio",    /* name of task. */
                          10000,          /* Stack size of task */
                          param,          /* parameter of the task */
                          0,              /* priority of the task */
                          &i2s_task,      /* Task handle to keep track of created task */
                          0);             /* pin task to core 0 */
}