#include "WiFi.h"
#include "audio.h"

NetworkClient client;

struct mixer {
  float (*proc)(uint64_t sample_count, mixer* data);
  float frequency;
};

float mix(uint64_t sample_count, mixer* data) {
  float dt = (float)sample_count / SAMPLE_RATE;
  return tsin(data->frequency * dt);
}

mixer my_mixer = {
  .proc = mix,
  .frequency = 0,
};

void setup() {
  Serial.begin(115200);
  Serial.println();

  const char* ssid = "esp_server";
  Serial.printf("Connecting to %s", ssid);
  WiFi.begin(ssid);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  init_audio(&my_mixer);
}

void loop() {
  if (!client) {
    static unsigned long last_attempt = 0;
    if (last_attempt == 0 || millis() > last_attempt + 5000) {
      Serial.println("connecting...");
      if (client.connect("192.168.4.1", 80)) {
        client.print("Hello\n");
        Serial.println("connected!");
      }
      last_attempt = millis();
    }
  }

  if (client) {
    if (client.available()) {
      String msg = client.readStringUntil('\n');
      my_mixer.frequency = msg.toFloat();
      Serial.println(msg);
    }
  }
}
